from typing import Optional

from pydantic import BaseModel, StrictStr

from .types import Datetime64


class StatementSchema(BaseModel):

    DisclosedDate: Datetime64
    DisclosedTime: Optional[StrictStr]
    LocalCode: StrictStr
    DisclosureNumber: StrictStr
    TypeOfDocument: StrictStr
    TypeOfCurrentPeriod: StrictStr
    CurrentPeriodStartDate: StrictStr
    CurrentPeriodEndDate: StrictStr
    CurrentFiscalYearEndDate: StrictStr
    CurrentFiscalYearStartDate: StrictStr
    NextFiscalYearStartDate: Optional[StrictStr]
    NextFiscalYearEndDate: Optional[StrictStr]
    NetSales: Optional[float]
    OperatingProfit: Optional[float]
    OrdinaryProfit: Optional[float]
    Profit: Optional[float]
    EarningsPerShare: Optional[float]
    DilutedEarningsPerShare: Optional[float]
    TotalAssets: Optional[float]
    Equity: Optional[float]
    EquityToAssetRatio: Optional[float]
    BookValuePerShare: Optional[float]
    CashFlowsFromOperatingActivities: Optional[float]
    CashFlowsFromInvestingActivities: Optional[float]
    CashFlowsFromFinancingActivities: Optional[float]
    CashAndEquivalents: Optional[float]
    ResultDividendPerShare1stQuarter: Optional[float]
    ResultDividendPerShare2ndQuarter: Optional[float]
    ResultDividendPerShare3rdQuarter: Optional[float]
    ResultDividendPerShareFiscalYearEnd: Optional[float]
    ResultDividendPerShareAnnual: Optional[float]
    DistributionsPerUnitREIT: Optional[float]
    ResultTotalDividendPaidAnnual: Optional[float]
    ResultPayoutRatioAnnual: Optional[float]
    ForecastDividendPerShare1stQuarter: Optional[float]
    ForecastDividendPerShare2ndQuarter: Optional[float]
    ForecastDividendPerShare3rdQuarter: Optional[float]
    ForecastDividendPerShareFiscalYearEnd: Optional[float]
    ForecastDividendPerShareAnnual: Optional[float]
    ForecastDistributionsPerUnitREIT: Optional[float]
    ForecastTotalDividendPaidAnnual: Optional[float]
    ForecastPayoutRatioAnnual: Optional[float]
    NextYearForecastDividendPerShare1stQuarter: Optional[float]
    NextYearForecastDividendPerShare2ndQuarter: Optional[float]
    NextYearForecastDividendPerShare3rdQuarter: Optional[float]
    NextYearForecastDividendPerShareFiscalYearEnd: Optional[float]
    NextYearForecastDividendPerShareAnnual: Optional[float]
    NextYearForecastDistributionsPerUnitREIT: Optional[float]
    NextYearForecastPayoutRatioAnnual: Optional[float]
    ForecastNetSales1ndQuarter: Optional[float]
    ForecastOperatingProfit1ndQuarter: Optional[float]
    ForecastOrdinaryProfit1ndQuarter: Optional[float]
    ForecastProfit1ndQuarter: Optional[float]
    ForecastEarningsPerShare2ndQuarter: Optional[float]
    NextYearForecastNetSales2ndQuarter: Optional[float]
    NextYearForecastOperatingProfit2ndQuarter: Optional[float]
    NextYearForecastOrdinaryProfit2ndQuarter: Optional[float]
    NextYearForecastProfit1ndQuarter: Optional[float]
    NextYearForecastEarningsPerShare2ndQuarter: Optional[float]
    ForecastNetSales: Optional[float]
    ForecastOperatingProfit: Optional[float]
    ForecastOrdinaryProfit: Optional[float]
    ForecastProfit: Optional[float]
    ForecastEarningsPerShare: Optional[float]
    NextYearForecastNetSales: Optional[float]
    NextYearForecastOperatingProfit: Optional[float]
    NextYearForecastOrdinaryProfit: Optional[float]
    NextYearForecastProfit: Optional[float]
    NextYearForecastEarningsPerShare: Optional[float]
    MaterialChangesInSubsidiaries: Optional[bool]
    ChangesBasedOnRevisionsOfAccountingStandard: Optional[bool]
    ChangesOtherThanOnesBasedOnRevisionsOfAccountingStandard: Optional[bool]
    ChangesInAccountingEstimates: Optional[bool]
    RetrospectiveRestatement: Optional[bool]
    NumberOfIssuedAndOutstandingSharesAtTheEndOfFiscalYearIncludingTreasuryStock: Optional[float]
    NumberOfTreasuryStockAtTheEndOfFiscalYear: Optional[float]
    AverageNumberOfShares: Optional[float]
    NonConsolidatedNetSales: Optional[float]
    NonConsolidatedOperatingProfit: Optional[float]
    NonConsolidatedOrdinaryProfit: Optional[float]
    NonConsolidatedProfit: Optional[float]
    NonConsolidatedEarningsPerShare: Optional[float]
    NonConsolidatedTotalAssets: Optional[float]
    NonConsolidatedEquity: Optional[float]
    NonConsolidatedEquityToAssetRatio: Optional[float]
    NonConsolidatedBookValuePerShare: Optional[float]
    ForecastNonConsolidatedNetSales2ndQuarter: Optional[float]
    ForecastNonConsolidatedOperatingProfit2ndQuarter: Optional[float]
    ForecastNonConsolidatedOrdinaryProfit2ndQuarter: Optional[float]
    ForecastNonConsolidatedProfit1ndQuarter: Optional[float]
    ForecastNonConsolidatedEarningsPerShare2ndQuarter: Optional[float]
    NextYearForecastNonConsolidatedNetSales2ndQuarter: Optional[float]
    NextYearForecastNonConsolidatedOperatingProfit2ndQuarter: Optional[float]
    NextYearForecastNonConsolidatedOrdinaryProfit2ndQuarter: Optional[float]
    NextYearForecastNonConsolidatedProfit2ndQuarter: Optional[float]
    NextYearForecastNonConsolidatedEarningsPerShare2ndQuarter: Optional[float]
    ForecastNonConsolidatedNetSales: Optional[float]
    ForecastNonConsolidatedOperatingProfit: Optional[float]
    ForecastNonConsolidatedOrdinaryProfit: Optional[float]
    ForecastNonConsolidatedProfit: Optional[float]
    ForecastNonConsolidatedEarningsPerShare: Optional[float]
    NextYearForecastNonConsolidatedNetSales: Optional[float]
    NextYearForecastNonConsolidatedOperatingProfit: Optional[float]
    NextYearForecastNonConsolidatedOrdinaryProfit: Optional[float]
    NextYearForecastNonConsolidatedProfit: Optional[float]
    NextYearForecastNonConsolidatedEarningsPerShare: Optional[float]
